@@ -409,12 +409,21 @@ incompleteGamma a x
             || (abs mu < 0.4)
     -- Gautschi's algorithm.
     --
-    -- Evaluate series for P(a,x). See [Temme1994] Eq. 5.5
-    --
-    -- FIXME: Term `exp (log x * z - x - logGamma (z+1))` doesn't give full precision
+    -- Evaluate series for P(a,x). See [Temme1994] Eq. 5.5 and [NOTE:
+    -- incompleteGamma.taylorP]
+    factorP
+      | a < 10     = x ** a
+                   / (exp x * exp (logGamma (a + 1)))
+      | a < 1182.5 = (x * exp 1 / a) ** a
+                   / exp x
+                   / sqrt (2*pi*a)
+                   / exp (logGammaCorrection a)
+      | otherwise  = (x * exp 1 / a * exp (-x/a)) ** a
+                   / sqrt (2*pi*a)
+                   / exp (logGammaCorrection a)
     taylorSeriesP
       = sumPowerSeries x (scanSequence (/) 1 $ enumSequenceFrom (a+1))
-      * exp (log x * a - x - logGamma (a+1))
+      * factorP
     -- Series for 1-Q(a,x). See [Temme1994] Eq. 5.5
     taylorSeriesComplQ
       = sumPowerSeries (-x) (scanSequence (/) 1 (enumSequenceFrom 1) / enumSequenceFrom a)
@@ -768,7 +777,7 @@ invIncBetaGuess beta a b p
     in x
   -- For small a and not too big b we use approximation from boost.
   | b < 5 && a <= 1 =
-    let x | p**(1/a) < 0.5 = (p * a * exp beta) ** (1/a)
+    let x | p**(1/a) < 0.5 = (p ** (a * exp beta)) ** (1/a)
           | otherwise      = 1 - (1 - p ** (b * exp beta))**(1/b)
     in x
   -- When a>>b and both are large approximation from [Temme1992],
@@ -1322,3 +1331,53 @@ factorialTable = U.fromListN 171
   , 4.269068009004705e304
   , 7.257415615307998e306
   ]
+
+
+-- [NOTE: incompleteGamma.taylorP]
+--
+-- Incompltete gamma uses several algorithms for different parts of
+-- parameter space. Most troublesome is P(a,x) Taylor series
+-- [Temme1994,Eq.5.5] which requires to evaluate rather nasty
+-- expression:
+--
+--       x^a             x^a
+--  ------------- = -------------
+--  exp(x)·Γ(a+1)   exp(x)·a·Γ(a)
+--
+--  Conditions:
+--    | 0.5<x<1.1  = x < 4/3*a
+--    | otherwise  = x < a
+--
+-- For small `a` computation could be performed directly. However for
+-- largish values of `a` it's possible some of factor in the
+-- expression overflow. Values below take into account ranges for
+-- Taylor P approximation:
+--
+--  · a > 155    - x^a could overflow
+--  · a > 1182.5 - exp(x) could overflow
+--
+-- Usual way to avoid overflow problem is to perform calculations in
+-- the log domain. It however doesn't work very well in this case
+-- since we encounter catastrophic cancellations and could easily lose
+-- up to 6(!) digits for large `a`.
+--
+-- So we take another approach and use Stirling approximation with
+-- correction (logGammaCorrection).
+--
+--              x^a               / x·e \^a         1
+--  ≈ ------------------------- = | --- | · ----------------
+--    exp(x)·sqrt(2πa)·(a/e)^a)   \  a  /   exp(x)·sqrt(2πa)
+--
+-- We're using this approach as soon as logGammaCorrection starts
+-- working (a>10) because we don't have implementation for gamma
+-- function and exp(logGamma z) results in errors for large a.
+--
+-- Once we get into region when exp(x) could overflow we rewrite
+-- expression above once more:
+--
+--  / x·e            \^a     1
+--  | --- · e^(-x/a) | · ---------
+--  \  a             /   sqrt(2πa)
+--
+-- This approach doesn't work very well but it's still big improvement
+-- over calculations in the log domain.
